@@ -51,6 +51,8 @@ mktime_from_string (char *date,
 	struct tm    tm = {0, };
 	char       **fields;
 
+	tm.tm_isdst = -1;
+
 	/* date */
 
 	fields = g_strsplit (date, ".", 3);
@@ -92,35 +94,57 @@ process_line (char     *line,
 	FrCommandAce  *ace_comm = FR_COMMAND_ACE (data);
 	FrCommand     *comm = FR_COMMAND (data);
 	char         **fields;
-	char          *field_name;
+	const char    *field_name;
 
 	g_return_if_fail (line != NULL);
 
+	if (ace_comm->command_type == FR_ACE_COMMAND_UNKNOWN) {
+		if (g_str_has_prefix (line, "UNACE")) {
+			if (strstr (line, "public version") != NULL)
+				ace_comm->command_type = FR_ACE_COMMAND_PUBLIC;
+			else
+				ace_comm->command_type = FR_ACE_COMMAND_NONFREE;
+		}
+		return;		
+	}
+
+	if (! ace_comm->list_started) {
+		if (ace_comm->command_type == FR_ACE_COMMAND_PUBLIC) {
+			if (g_str_has_prefix (line, "Date"))
+				ace_comm->list_started = TRUE;
+		}
+		else if (ace_comm->command_type == FR_ACE_COMMAND_NONFREE) {
+			if (g_str_has_prefix (line, "  Date"))
+				ace_comm->list_started = TRUE;
+		}
+		return;
+	}
+
 	fdata = file_data_new ();
 
-	fields = g_strsplit (line, "|", 6);
+	if (ace_comm->command_type == FR_ACE_COMMAND_PUBLIC)
+		fields = g_strsplit (line, "|", 6);
+	else if (ace_comm->command_type == FR_ACE_COMMAND_NONFREE)
+		fields = split_line (line, 5);
 
-	if ((fields == NULL) || (fields[0] == NULL))
+	if ((fields == NULL) || (fields[0] == NULL) || (n_fields (fields) < 5))
 		return;
 
-	if (! ace_comm->list_started && (strncmp(fields[0], "Date", 4) == 0)) {
-		ace_comm->list_started = TRUE;
-		return;
-	}
-
-	if (n_fields (fields) != 6) {
-		ace_comm->list_started = FALSE;
-		return;
-	}
-
 	fdata->size = g_ascii_strtoull (fields[3], NULL, 10);
 	fdata->modified = mktime_from_string (fields[0], fields[1]);
 
-	field_name = fields[5] + 1;
+	if (ace_comm->command_type == FR_ACE_COMMAND_PUBLIC) {
+		field_name = fields[5];
+		field_name = field_name + 1;
+	}
+	else if (ace_comm->command_type == FR_ACE_COMMAND_NONFREE)
+		field_name = get_last_field (line, 6);
+
 	if (field_name[0] != '/') {
 		fdata->full_path = g_strconcat ("/", field_name, NULL);
 		fdata->original_path = fdata->full_path + 1;
-	} else {
+	} 
+	else {
 		fdata->full_path = g_strdup (field_name);
 		fdata->original_path = fdata->full_path;
 	}
@@ -133,7 +157,7 @@ process_line (char     *line,
 	if (*fdata->name == 0)
 		file_data_free (fdata);
 	else 
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 }
 
 
@@ -142,6 +166,7 @@ fr_command_ace_list (FrCommand  *comm,
 		    const char *password)
 {
 	FR_COMMAND_ACE (comm)->list_started = FALSE;
+	FR_COMMAND_ACE (comm)->command_type = FR_ACE_COMMAND_UNKNOWN;
 
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
 				      process_line,
@@ -149,7 +174,8 @@ fr_command_ace_list (FrCommand  *comm,
 
 	fr_process_begin_command (comm->process, "unace");
 	fr_process_add_arg (comm->process, "v");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, "-y");
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
@@ -175,7 +201,8 @@ fr_command_ace_extract (FrCommand   *comm,
 		fr_process_add_arg (comm->process, "e");
 	else
 		fr_process_add_arg (comm->process, "x");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, "-y");
+	fr_process_add_arg (comm->process, comm->filename);
 
 	for (scan = file_list; scan; scan = scan->next)
 		fr_process_add_arg (comm->process, scan->data);
@@ -190,19 +217,32 @@ fr_command_ace_test (FrCommand   *comm,
 {
         fr_process_begin_command (comm->process, "unace");
         fr_process_add_arg (comm->process, "t");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, "-y");
+	fr_process_add_arg (comm->process, comm->filename);
         fr_process_end_command (comm->process);
 }
 
 
 static void
 fr_command_ace_handle_error (FrCommand   *comm, 
-			     FRProcError *error)
+			     FrProcError *error)
 {
 	/* FIXME */
 }
 
 
+static void
+fr_command_ace_set_mime_type (FrCommand  *comm,
+			      const char *mime_type)
+{
+	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
+	
+	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("unace")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ;
+}
+
+
 static void 
 fr_command_ace_class_init (FrCommandAceClass *class)
 {
@@ -218,15 +258,13 @@ fr_command_ace_class_init (FrCommandAceClass *class)
 	afc->extract        = fr_command_ace_extract;
 	afc->test           = fr_command_ace_test;
 	afc->handle_error   = fr_command_ace_handle_error;
+	afc->set_mime_type  = fr_command_ace_set_mime_type;
 }
 
  
 static void 
 fr_command_ace_init (FrCommand *comm)
 {
-	comm->file_type = FR_FILE_TYPE_ACE;
-
-	comm->propCanModify                = FALSE;
 	comm->propAddCanUpdate             = TRUE;
 	comm->propAddCanReplace            = TRUE;
 	comm->propExtractCanAvoidOverwrite = FALSE;
@@ -275,19 +313,3 @@ fr_command_ace_get_type ()
 
         return type;
 }
-
-
-FrCommand *
-fr_command_ace_new (FrProcess  *process,
-		    const char *filename)
-{
-	FrCommand *comm;
-
-	if(! is_program_in_path ("unace")) 
-		return NULL;
-	
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_ACE, NULL));
-	fr_command_construct (comm, process, filename);
-	
-	return comm;
-}
