@@ -53,6 +53,8 @@ mktime_from_string (char *month,
 				   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 	struct tm     tm = {0, };
 
+	tm.tm_isdst = -1;
+
 	if (month != NULL) {
 		int i;
 		for (i = 0; i < 12; i++)
@@ -68,7 +70,7 @@ mktime_from_string (char *month,
 	        	time_t      now;
         		struct tm  *now_tm;
 
-	  		tm.tm_hour = atoi (fields[0]) - 1;
+	  		tm.tm_hour = atoi (fields[0]);
 	  		tm.tm_min = atoi (fields[1]);
 	  
 	  		now = time(NULL);
@@ -139,7 +141,7 @@ list__process_line (char     *line,
 	if (*fdata->name == 0)
 		file_data_free (fdata);
 	else 
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 }
 
 
@@ -147,12 +149,20 @@ static void
 fr_command_cpio_list (FrCommand  *comm,
 		      const char *password)
 {
+	GString *cmd;
+	
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
 				      list__process_line,
 				      comm);
 
-	fr_process_begin_command (comm->process, "cpio -itv <");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_begin_command (comm->process, "sh");
+	fr_process_add_arg (comm->process, "-c");
+	
+	cmd = g_string_new ("cpio -itv < ");
+	g_string_append (cmd, comm->e_filename);
+	fr_process_add_arg (comm->process, cmd->str);
+	g_string_free (cmd, TRUE);
+	
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
@@ -167,22 +177,43 @@ fr_command_cpio_extract (FrCommand  *comm,
 			gboolean    junk_paths,
 			const char *password)
 {
-	GList *scan;
+	GList   *scan;
+	GString *cmd;
 
-	fr_process_begin_command (comm->process, "cpio -idu");
+	fr_process_begin_command (comm->process, "sh");
+	fr_process_add_arg (comm->process, "-c");
 	if (dest_dir != NULL)
                 fr_process_set_working_dir (comm->process, dest_dir);
+	
+	cmd = g_string_new ("cpio -idu ");
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = (char*) scan->data;
-		fr_process_add_arg (comm->process, filename);
+		char *filename = g_shell_quote (scan->data); 
+		g_string_append (cmd, filename);
+		g_free (filename);
+		g_string_append (cmd, " ");
 	}
-        fr_process_add_arg (comm->process, "<");
-	fr_process_add_arg (comm->process, comm->e_filename);
+        g_string_append (cmd, " < ");
+	g_string_append (cmd, comm->e_filename);
+	fr_process_add_arg (comm->process, cmd->str);
+	g_string_free (cmd, TRUE);
+	
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
 
 
+static void
+fr_command_cpio_set_mime_type (FrCommand  *comm,
+		 	       const char *mime_type)
+{
+	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
+	
+	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("cpio")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ;
+}
+
+
 static void 
 fr_command_cpio_class_init (FrCommandCpioClass *class)
 {
@@ -196,15 +227,13 @@ fr_command_cpio_class_init (FrCommandCpioClass *class)
 
         afc->list           = fr_command_cpio_list;
 	afc->extract        = fr_command_cpio_extract;
+	afc->set_mime_type  = fr_command_cpio_set_mime_type;
 }
 
  
 static void 
 fr_command_cpio_init (FrCommand *comm)
 {
-	comm->file_type = FR_FILE_TYPE_CPIO;
-
-	comm->propCanModify                = FALSE;
 	comm->propAddCanUpdate             = FALSE;
 	comm->propAddCanReplace            = FALSE;
 	comm->propAddCanStoreFolders       = FALSE;
@@ -254,20 +283,3 @@ fr_command_cpio_get_type ()
 
         return type;
 }
-
-
-FrCommand *
-fr_command_cpio_new (FrProcess  *process,
-		    const char *filename)
-{
-	FrCommand *comm;
-
-	if (!is_program_in_path("cpio")) {
-		return NULL;
-	}
-
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_CPIO, NULL));
-	fr_command_construct (comm, process, filename);
-
-	return comm;
-}
