@@ -178,18 +178,8 @@ add_password_arg (FrCommand  *comm,
 		  const char *password,
 		  gboolean    disable_query)
 {
-	if ((password != NULL) && (password[0] != '\0')) {
-		char *arg;
-		char *quoted_arg;
-		
-		arg = g_strdup_printf ("-p%s", password);
-		quoted_arg = g_shell_quote (arg);
-
-		fr_process_add_arg (comm->process, quoted_arg);
-
-		g_free (quoted_arg);
-		g_free (arg);
-	} 
+	if ((password != NULL) && (password[0] != '\0'))
+		fr_process_add_arg_concat (comm->process, "-p", password, NULL); 
 	else if (disable_query)
 		fr_process_add_arg (comm->process, "-p-");
 }
@@ -217,7 +207,7 @@ fr_command_rar_list (FrCommand  *comm,
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
@@ -228,8 +218,9 @@ fr_command_rar_add (FrCommand     *comm,
 		    GList         *file_list,
 		    const char    *base_dir,
 		    gboolean       update,
+		    gboolean       recursive,
 		    const char    *password,
-		    FRCompression  compression)
+		    FrCompression  compression)
 {
 	GList *scan;
 
@@ -262,10 +253,10 @@ fr_command_rar_add (FrCommand     *comm,
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
 	for (scan = file_list; scan; scan = scan->next)
-		fr_process_add_arg (comm->process, (gchar*) scan->data);
+		fr_process_add_arg (comm->process, scan->data);
 
 	fr_process_end_command (comm->process);
 }
@@ -283,7 +274,7 @@ fr_command_rar_delete (FrCommand *comm,
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
 	for (scan = file_list; scan; scan = scan->next)
 		fr_process_add_arg (comm->process, scan->data);
@@ -328,16 +319,13 @@ fr_command_rar_extract (FrCommand  *comm,
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 
 	for (scan = file_list; scan; scan = scan->next)
 		fr_process_add_arg (comm->process, scan->data);
 
-	if (dest_dir != NULL) {
-		char *e_dest_dir = fr_command_escape (comm, dest_dir);
-		fr_process_add_arg (comm->process, e_dest_dir);
-		g_free (e_dest_dir);
-	}
+	if (dest_dir != NULL)
+		fr_process_add_arg (comm->process, dest_dir);
 
 	fr_process_end_command (comm->process);
 }
@@ -362,36 +350,53 @@ fr_command_rar_test (FrCommand   *comm,
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--");
 
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
 }
 
 
 static void
 fr_command_rar_handle_error (FrCommand   *comm,
-			     FRProcError *error)
+			     FrProcError *error)
 {
-	if (error->type == FR_PROC_ERROR_COMMAND_ERROR) {
+	if (error->type != FR_PROC_ERROR_COMMAND_ERROR) 
+		return;
+		
+	if (error->status == 3) {
+		error->type = FR_PROC_ERROR_ASK_PASSWORD;
+	}
+	else {
+		GList *scan;
+		
 		if (error->status <= 1)
 			error->type = FR_PROC_ERROR_NONE;
-		else if (error->status == 3)
-			error->type = FR_PROC_ERROR_ASK_PASSWORD;
-		else {
-			GList *scan;
 			
-			for (scan = g_list_last (comm->process->raw_error); scan; scan = scan->prev) {
-				char *line = scan->data;
+		for (scan = g_list_last (comm->process->err.raw); scan; scan = scan->prev) {
+			char *line = scan->data;
 				
-				if (strstr (line, "password incorrect") != NULL) {
-					error->type = FR_PROC_ERROR_ASK_PASSWORD;
-					break;
-				}
+			if (strstr (line, "password incorrect") != NULL) {
+				error->type = FR_PROC_ERROR_ASK_PASSWORD;
+				break;
 			}
 		}
 	}
 }
 
 
+static void
+fr_command_rar_set_mime_type (FrCommand  *comm,
+		 	      const char *mime_type)
+{
+	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
+	
+	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("rar")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+	else if (is_program_in_path ("unrar")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ;
+}
+
+
 static void
 fr_command_rar_class_init (FrCommandRarClass *class)
 {
@@ -403,20 +408,19 @@ fr_command_rar_class_init (FrCommandRarClass *class)
 
 	gobject_class->finalize = fr_command_rar_finalize;
 
-	afc->list         = fr_command_rar_list;
-	afc->add          = fr_command_rar_add;
-	afc->delete       = fr_command_rar_delete;
-	afc->extract      = fr_command_rar_extract;
-	afc->test         = fr_command_rar_test;
-	afc->handle_error = fr_command_rar_handle_error;
+	afc->list           = fr_command_rar_list;
+	afc->add            = fr_command_rar_add;
+	afc->delete         = fr_command_rar_delete;
+	afc->extract        = fr_command_rar_extract;
+	afc->test           = fr_command_rar_test;
+	afc->handle_error   = fr_command_rar_handle_error;
+	afc->set_mime_type  = fr_command_rar_set_mime_type;
 }
 
 
 static void
 fr_command_rar_init (FrCommand *comm)
 {
-	comm->file_type = FR_FILE_TYPE_RAR;
-
 	comm->propAddCanUpdate             = TRUE;
 	comm->propAddCanReplace            = TRUE;
 	comm->propAddCanStoreFolders       = TRUE;
@@ -466,19 +470,3 @@ fr_command_rar_get_type ()
 
 	return type;
 }
-
-
-FrCommand *
-fr_command_rar_new (FrProcess  *process,
-		    const char *filename)
-{
-	FrCommand *comm;
-
-	if ((!is_program_in_path("rar")) && (!is_program_in_path("unrar")))
-		return NULL;
-
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_RAR, NULL));
-	fr_command_construct (comm, process, filename);
-
-	return comm;
-}
