@@ -124,15 +124,21 @@ list__process_line (char     *line,
 	}
 
 	if (*(fields[0]) == '/') {
-		fdata->full_path = g_strdup (fields[0]);
+		fdata->full_path = g_strcompress (fields[0]);
 		fdata->original_path = fdata->full_path;
-	} else {
-		fdata->full_path = g_strconcat ("/", fields[0], NULL);
+	} 
+	else {
+		char *compressed;
+		
+		compressed = g_strcompress (fields[0]);
+		fdata->full_path = g_strconcat ("/", compressed, NULL);
+		g_free (compressed);
+		
 		fdata->original_path = fdata->full_path + 1;
 	}
 
 	if (fields[1] != NULL)
-		fdata->link = g_strdup (fields[1]);
+		fdata->link = g_strcompress (fields[1]);
 	g_strfreev (fields);
 
 	fdata->name = g_strdup (file_name_from_path (fdata->full_path));
@@ -153,9 +159,9 @@ fr_command_rpm_list (FrCommand  *comm,
 				      list__process_line,
 				      comm);
 
-	fr_process_begin_command (comm->process, "rpm2cpio");
-	fr_process_add_arg (comm->process, comm->e_filename);
-	fr_process_add_arg (comm->process, "| cpio -itv");
+	fr_process_begin_command (comm->process, "sh");
+	fr_process_add_arg (comm->process, "-c");
+	fr_process_add_arg_concat (comm->process, "rpm2cpio ", comm->e_filename, " | cpio -itv", NULL);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
@@ -170,22 +176,43 @@ fr_command_rpm_extract (FrCommand  *comm,
 			gboolean    junk_paths,
 			const char *password)
 {
-	GList *scan;
-
-	fr_process_begin_command (comm->process, "rpm2cpio");
+	GList   *scan;
+	GString *cmd;
+	
+	fr_process_begin_command (comm->process, "sh");
 	if (dest_dir != NULL)
                 fr_process_set_working_dir (comm->process, dest_dir);
-	fr_process_add_arg (comm->process, comm->e_filename);
-	fr_process_add_arg (comm->process, "| cpio -idu");
+	fr_process_add_arg (comm->process, "-c");
+	
+	cmd = g_string_new ("rpm2cpio ");
+	g_string_append (cmd, comm->e_filename);
+	g_string_append (cmd, " | cpio -idu ");
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = (char*) scan->data;
-		fr_process_add_arg (comm->process, filename);
+		char *filename = g_shell_quote (scan->data); 
+		g_string_append (cmd, filename);
+		g_free (filename);
+		g_string_append (cmd, " ");
 	}
+	fr_process_add_arg (comm->process, cmd->str);
+	g_string_free (cmd, TRUE);
+	
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
 }
 
 
+static void
+fr_command_rpm_set_mime_type (FrCommand  *comm,
+		 	      const char *mime_type)
+{
+	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
+	
+	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("rpm2cpio")) 
+		comm->capabilities |= FR_COMMAND_CAP_READ;
+}
+
+
 static void 
 fr_command_rpm_class_init (FrCommandRpmClass *class)
 {
@@ -199,15 +226,13 @@ fr_command_rpm_class_init (FrCommandRpmClass *class)
 
         afc->list           = fr_command_rpm_list;
 	afc->extract        = fr_command_rpm_extract;
+	afc->set_mime_type  = fr_command_rpm_set_mime_type;
 }
 
  
 static void 
 fr_command_rpm_init (FrCommand *comm)
 {
-	comm->file_type = FR_FILE_TYPE_RPM;
-
-	comm->propCanModify                = FALSE;
 	comm->propAddCanUpdate             = FALSE;
 	comm->propAddCanReplace            = FALSE;
 	comm->propExtractCanAvoidOverwrite = FALSE;
@@ -256,20 +281,3 @@ fr_command_rpm_get_type ()
 
         return type;
 }
-
-
-FrCommand *
-fr_command_rpm_new (FrProcess  *process,
-		    const char *filename)
-{
-	FrCommand *comm;
-
-	if (!is_program_in_path("rpm2cpio")) {
-		return NULL;
-	}
-
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_RPM, NULL));
-	fr_command_construct (comm, process, filename);
-
-	return comm;
-}
