@@ -133,10 +133,12 @@ process_line (char     *line,
 
 	g_return_if_fail (line != NULL);
 
-	fdata = file_data_new ();
-
 	date_idx = file_list__get_index_from_pattern (line, "%n%n%n%n-%n%n-%n%n %n%n:%n%n");
+	if (date_idx < 0)
+		return;
 
+	fdata = file_data_new ();
+		
 	field_size = file_list__get_prev_field (line, date_idx, 1);
 	fdata->size = g_ascii_strtoull (field_size, NULL, 10);
 	g_free (field_size);
@@ -157,7 +159,7 @@ process_line (char     *line,
 		fields = g_strsplit (field_name, " link to ", 2);
 	}
 
-	name = unescape_str (fields[0]);
+	name = g_strcompress (fields[0]);
 	if (*name == '/') {
 		fdata->full_path = g_strdup (name);
 		fdata->original_path = fdata->full_path;
@@ -191,14 +193,26 @@ add_compress_arg (FrCommand *comm)
 {
 	if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) 
 		fr_process_add_arg (comm->process, "-z");
+		
 	else if (is_mime_type (comm->mime_type, "application/x-bzip-compressed-tar"))
 		fr_process_add_arg (comm->process, "--use-compress-program=bzip2");
-	else if (is_mime_type (comm->mime_type, "application/x-compressed-tar"))
+		
+	else if (is_mime_type (comm->mime_type, "application/x-tarz"))
 		fr_process_add_arg (comm->process, "-Z");
+		
 	else if (is_mime_type (comm->mime_type, "application/x-lzma-compressed-tar"))
 		fr_process_add_arg (comm->process, "--use-compress-program=lzma");
+		
 	else if (is_mime_type (comm->mime_type, "application/x-lzop-compressed-tar"))
 		fr_process_add_arg (comm->process, "--use-compress-program=lzop");
+		
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		FrCommandTar *comm_tar = (FrCommandTar*) comm;
+		char         *option;	
+		option = g_strdup_printf ("--use-compress-program=%s", comm_tar->compress_command);
+		fr_process_add_arg (comm->process, option);
+		g_free (option);
+	}
 }
 
 
@@ -234,7 +248,7 @@ fr_command_tar_list (FrCommand  *comm,
 	fr_process_add_arg (comm->process, "--force-local");
 	fr_process_add_arg (comm->process, "--no-wildcards");
 	fr_process_add_arg (comm->process, "-tvf");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	add_compress_arg (comm);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
@@ -279,6 +293,7 @@ fr_command_tar_add (FrCommand     *comm,
 		    GList         *file_list,
 		    const char    *base_dir,
 		    gboolean       update,
+		    gboolean       recursive,
 		    const char    *password,
 		    FrCompression  compression)
 {
@@ -291,16 +306,15 @@ fr_command_tar_add (FrCommand     *comm,
 
 	begin_tar_command (comm);
 	fr_process_add_arg (comm->process, "--force-local");
-	fr_process_add_arg (comm->process, "--no-recursion");
+	if (! recursive)
+		fr_process_add_arg (comm->process, "--no-recursion");
 	fr_process_add_arg (comm->process, "--no-wildcards");
 	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "-p");
 		
 	if (base_dir != NULL) {
-		char *e_base_dir = shell_escape (base_dir);
 		fr_process_add_arg (comm->process, "-C");
-		fr_process_add_arg (comm->process, e_base_dir);
-		g_free (e_base_dir);
+		fr_process_add_arg (comm->process, base_dir);
 	}
 
 	fr_process_add_arg (comm->process, "-rf");
@@ -392,14 +406,12 @@ fr_command_tar_extract (FrCommand  *comm,
 		fr_process_add_arg (comm->process, "--keep-newer-files");
 		
 	fr_process_add_arg (comm->process, "-xf");
-	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	add_compress_arg (comm);
 
 	if (dest_dir != NULL) {
-		char *e_dest_dir = shell_escape (dest_dir);
 		fr_process_add_arg (comm->process, "-C");
-		fr_process_add_arg (comm->process, e_dest_dir);
-		g_free (e_dest_dir);
+		fr_process_add_arg (comm->process, dest_dir);
 	}
 	
 	fr_process_add_arg (comm->process, "--");
@@ -466,7 +478,7 @@ fr_command_tar_recompress (FrCommand     *comm,
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".bz2", NULL);
 	}
-	else if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
+	else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
 		fr_process_begin_command (comm->process, "compress");
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
@@ -517,6 +529,40 @@ fr_command_tar_recompress (FrCommand     *comm,
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".lzo", NULL);
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		FrCommandTar *comm_tar = (FrCommandTar*) comm;
+		
+		fr_process_begin_command (comm->process, comm_tar->compress_command);
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
+		switch (compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-mx=1"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-mx=5"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-mx=5"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-mx=7"); break;
+		}
+		fr_process_add_arg (comm->process, "a");
+		fr_process_add_arg (comm->process, "-bd");
+		fr_process_add_arg (comm->process, "-y");
+		fr_process_add_arg (comm->process, "-l");
+		
+		new_name = g_strconcat (c_tar->uncomp_filename, ".7z", NULL);
+		fr_process_add_arg_concat (comm->process, new_name);
+		
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
+
+		/* remove the uncompressed tar */
+		
+		fr_process_begin_command (comm->process, "rm");
+		fr_process_add_arg (comm->process, "-f");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
+	}
 
 	if (c_tar->name_modified) {
 		char *tmp_dir;
@@ -527,7 +573,7 @@ fr_command_tar_recompress (FrCommand     *comm,
 		fr_process_set_sticky (comm->process, TRUE);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, new_name);
-		fr_process_add_arg (comm->process, comm->e_filename);
+		fr_process_add_arg (comm->process, comm->filename);
 		fr_process_end_command (comm->process);
 		
 		tmp_dir = remove_level_from_path (new_name);
@@ -585,7 +631,7 @@ get_uncompressed_name (FrCommandTar *c_tar,
 		else if (file_extension_is (e_filename, ".tar.bz2"))
 			new_name[l - 4] = 0;
 	}
-	else if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
+	else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
 		/* X.taz   -->  X.tar
 		 * X.tar.Z -->  X.tar */
 		if (file_extension_is (e_filename, ".taz"))
@@ -609,6 +655,11 @@ get_uncompressed_name (FrCommandTar *c_tar,
 		else if (file_extension_is (e_filename, ".tar.lzo"))
 			new_name[l - 4] = 0;
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		/* X.tar.7z -->  X.tar */
+		if (file_extension_is (e_filename, ".tar.7z")) 
+			new_name[l - 3] = 0;
+	}
 
 	return new_name;
 }
@@ -625,16 +676,13 @@ get_temp_name (FrCommandTar *c_tar,
 	char *template;	
 	char *result = NULL;
 	char *temp_name = NULL;
-	char *e_temp_name = NULL;
 
 	template = g_strconcat (dirname, "/.fr-XXXXXX", NULL);
 	result = mkdtemp (template);
 	temp_name = g_build_filename (result, file_name_from_path (filepath), NULL);
-	e_temp_name = shell_escape (temp_name);
-	g_free (temp_name);
 	g_free (template);
 
-	return e_temp_name;
+	return temp_name;
 }
 
 
@@ -662,13 +710,13 @@ fr_command_tar_uncompress (FrCommand *comm)
 		if (archive_exists) {
 			fr_process_begin_command (comm->process, "mv");
 			fr_process_add_arg (comm->process, "-f");
-			fr_process_add_arg (comm->process, comm->e_filename);
+			fr_process_add_arg (comm->process, comm->filename);
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
 	} 
 	else
-		tmp_name = g_strdup (comm->e_filename);
+		tmp_name = g_strdup (comm->filename);
 
 	if (archive_exists) {
 		if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) { 
@@ -687,7 +735,7 @@ fr_command_tar_uncompress (FrCommand *comm)
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
-		else if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
+		else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
 			fr_process_begin_command (comm->process, "uncompress");
 			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
@@ -710,6 +758,24 @@ fr_command_tar_uncompress (FrCommand *comm)
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
+		else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+			FrCommandTar *comm_tar = (FrCommandTar*) comm;
+			
+			fr_process_begin_command (comm->process, comm_tar->compress_command);
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_add_arg (comm->process, "e");
+			fr_process_add_arg (comm->process, "-bd");
+			fr_process_add_arg (comm->process, "-y");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+			
+			/* remove the compressed tar */
+			
+			fr_process_begin_command (comm->process, "rm");
+			fr_process_add_arg (comm->process, "-f");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+		}
 	}
 
 	c_tar->uncomp_filename = get_uncompressed_name (c_tar, tmp_name);
@@ -717,22 +783,6 @@ fr_command_tar_uncompress (FrCommand *comm)
 }
 
 
-static char *
-fr_command_tar_escape (FrCommand     *comm,
-		       const char    *str)
-{
-	char *estr;
-	char *estr2;
-
-        estr = escape_str (str, "\\");
-        estr2 = escape_str (estr, "\\$?*'`& !|()@#:;<>");
-
-	g_free (estr);
-
-	return estr2;
-}
-
-
 static void
 fr_command_tar_handle_error (FrCommand   *comm,
 			     FrProcError *error)
@@ -748,6 +798,8 @@ static void
 fr_command_tar_set_mime_type (FrCommand  *comm,
 		 	      const char *mime_type)
 {
+	FrCommandTar *comm_tar = FR_COMMAND_TAR (comm);
+	
 	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
 
 	/* In solaris gtar is present under /usr/sfw/bin */
@@ -768,7 +820,7 @@ fr_command_tar_set_mime_type (FrCommand  *comm,
 		if (is_program_in_path ("bzip2"))
 			comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
 	}
-	else if (is_mime_type (comm->mime_type, "application/x-compressed-tar")) {
+	else if (is_mime_type (comm->mime_type, "application/x-tarz")) {
 		if (is_program_in_path ("compress"))
 			comm->capabilities |= FR_COMMAND_CAP_WRITE;
 		if (is_program_in_path ("uncompress"))
@@ -782,6 +834,18 @@ fr_command_tar_set_mime_type (FrCommand  *comm,
 		if (is_program_in_path ("lzop"))
 			comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-7z-compressed-tar")) {
+		char *try_command[3] = { "7za", "7zr", "7z" };
+		int   i;
+		
+		for (i = 0; i < G_N_ELEMENTS (try_command); i++) {
+			if (is_program_in_path (try_command[i])) {
+				comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+				comm_tar->compress_command = g_strdup (try_command[i]);
+				break;
+			}
+		}
+	}
 }
 
 
@@ -805,8 +869,6 @@ fr_command_tar_class_init (FrCommandTarClass *class)
 
 	afc->recompress     = fr_command_tar_recompress;
 	afc->uncompress     = fr_command_tar_uncompress;
-
-	afc->escape         = fr_command_tar_escape;
 }
 
 
@@ -815,11 +877,10 @@ fr_command_tar_init (FrCommand *comm)
 {
 	FrCommandTar *comm_tar = (FrCommandTar*) comm;
 
-	comm->propCanModify                 = TRUE;
 	comm->propAddCanUpdate              = FALSE;
 	comm->propAddCanReplace             = FALSE;
 	comm->propAddCanStoreFolders        = TRUE;
-	comm->propExtractCanAvoidOverwrite  = TRUE;
+	comm->propExtractCanAvoidOverwrite  = FALSE;
 	comm->propExtractCanSkipOlder       = TRUE;
 	comm->propExtractCanJunkPaths       = FALSE;
 	comm->propPassword                  = FALSE;
@@ -851,6 +912,11 @@ fr_command_tar_finalize (GObject *object)
 		g_free (comm_tar->msg);
 		comm_tar->msg = NULL;
 	}
+	
+	if (comm_tar->compress_command != NULL) {
+		g_free (comm_tar->compress_command);
+		comm_tar->compress_command = NULL;
+	}
 
 	/* Chain up */
         if (G_OBJECT_CLASS (parent_class)->finalize)
