@@ -22,6 +22,8 @@
 
 #include <config.h>
 #include <string.h>
+#include <sys/types.h>
+#include <signal.h>
 
 #include <gio/gio.h>
 #include <glade/glade.h>
@@ -77,36 +79,37 @@ static int    extract_here;
 static char  *default_url = NULL;
 
 FrMimeTypeDescription mime_type_desc[] = {
-	{ "application/x-ace",                 ".ace",      N_("Ace (.ace)"), FALSE, FALSE },
-	{ "application/x-ar",                  ".ar",       N_("Ar (.ar)"), FALSE, TRUE },
-	{ "application/x-arj",                 ".arj",      N_("Arj (.arj)"), TRUE, TRUE },
-	{ "application/x-bzip",                ".bz2",      NULL, FALSE, FALSE },
-	{ "application/x-bzip1",               ".bz",       NULL, FALSE, FALSE },
-	{ "application/x-compress",            ".Z",        NULL, FALSE, FALSE },
-	{ "application/x-cpio",                ".cpio",     NULL, FALSE, TRUE },
-	{ "application/x-deb",                 ".deb",      NULL, FALSE, TRUE },
-	{ "application/x-cd-image",            ".iso",      NULL, FALSE, TRUE },
-	{ "application/x-ear",                 ".ear",      N_("Ear (.ear)"), TRUE, TRUE },
-	{ "application/x-ms-dos-executable",   ".exe",      N_("Self-extracting zip (.exe)"), FALSE, TRUE },
-	{ "application/x-gzip",                ".gz",       NULL, FALSE, FALSE },
-	{ "application/x-jar",                 ".jar",      N_("Jar (.jar)"), TRUE, TRUE },
-	{ "application/x-lha",                 ".lzh",      N_("Lha (.lzh)"), FALSE, TRUE },
-	{ "application/x-lzma",                ".lzma",     NULL, FALSE, FALSE },
-	{ "application/x-lzop",                ".lzo",      NULL, FALSE, FALSE },
-	{ "application/x-rar",                 ".rar",      N_("Rar (.rar)"), TRUE, TRUE },
-	{ "application/x-rpm",                 ".rpm",      NULL, FALSE, TRUE },
-	{ "application/x-tar",                 ".tar",      N_("Tar uncompressed (.tar)"), FALSE, TRUE },
+	{ "application/x-ace",                  ".ace",      N_("Ace (.ace)"), FALSE, FALSE },
+	{ "application/x-ar",                   ".ar",       N_("Ar (.ar)"), FALSE, TRUE },
+	{ "application/x-arj",                  ".arj",      N_("Arj (.arj)"), TRUE, TRUE },
+	{ "application/x-bzip",                 ".bz2",      NULL, FALSE, FALSE },
+	{ "application/x-bzip1",                ".bz",       NULL, FALSE, FALSE },
+	{ "application/x-compress",             ".Z",        NULL, FALSE, FALSE },
+	{ "application/x-cpio",                 ".cpio",     NULL, FALSE, TRUE },
+	{ "application/x-deb",                  ".deb",      NULL, FALSE, TRUE },
+	{ "application/x-cd-image",             ".iso",      NULL, FALSE, TRUE },
+	{ "application/x-ear",                  ".ear",      N_("Ear (.ear)"), TRUE, TRUE },
+	{ "application/x-ms-dos-executable",    ".exe",      N_("Self-extracting zip (.exe)"), FALSE, TRUE },
+	{ "application/x-gzip",                 ".gz",       NULL, FALSE, FALSE },
+	{ "application/x-jar",                  ".jar",      N_("Jar (.jar)"), TRUE, TRUE },
+	{ "application/x-lha",                  ".lzh",      N_("Lha (.lzh)"), FALSE, TRUE },
+	{ "application/x-lzma",                 ".lzma",     NULL, FALSE, FALSE },
+	{ "application/x-lzop",                 ".lzo",      NULL, FALSE, FALSE },
+	{ "application/x-rar",                  ".rar",      N_("Rar (.rar)"), TRUE, TRUE },
+	{ "application/x-rpm",                  ".rpm",      NULL, FALSE, TRUE },
+	{ "application/x-tar",                  ".tar",      N_("Tar uncompressed (.tar)"), FALSE, TRUE },
 	{ "application/x-bzip1-compressed-tar", ".tar.bz",   N_("Tar compressed with bzip (.tar.bz)"), FALSE, TRUE },
-	{ "application/x-bzip-compressed-tar", ".tar.bz2",  N_("Tar compressed with bzip2 (.tar.bz2)"), FALSE, TRUE },
-	{ "application/x-compressed-tar",      ".tar.gz",   N_("Tar compressed with gzip (.tar.gz)"), FALSE, TRUE },
-	{ "application/x-lzma-compressed-tar", ".tar.lzma", N_("Tar compressed with lzma (.tar.lzma)"), FALSE, TRUE },
-	{ "application/x-lzop-compressed-tar", ".tar.lzo",  N_("Tar compressed with lzop (.tar.lzo)"), FALSE, TRUE },
-	{ "application/x-compressed-tar",      ".tar.Z",    N_("Tar compressed with compress (.tar.Z)"), FALSE, TRUE },
-	{ "application/x-stuffit",             ".sit",      NULL, FALSE, TRUE },
-	{ "application/x-war",                 ".war",      N_("War (.war)"), TRUE, TRUE },
-	{ "application/zip",                   ".zip",      N_("Zip (.zip)"), TRUE, TRUE },
-	{ "application/x-zoo",                 ".zoo",      N_("Zoo (.zoo)"), FALSE, TRUE },
-	{ "application/x-7z-compressed",       ".7z",       N_("7-Zip (.7z)"), TRUE, TRUE },
+	{ "application/x-bzip-compressed-tar",  ".tar.bz2",  N_("Tar compressed with bzip2 (.tar.bz2)"), FALSE, TRUE },
+	{ "application/x-compressed-tar",       ".tar.gz",   N_("Tar compressed with gzip (.tar.gz)"), FALSE, TRUE },
+	{ "application/x-lzma-compressed-tar",  ".tar.lzma", N_("Tar compressed with lzma (.tar.lzma)"), FALSE, TRUE },
+	{ "application/x-lzop-compressed-tar",  ".tar.lzo",  N_("Tar compressed with lzop (.tar.lzo)"), FALSE, TRUE },
+	{ "application/x-7z-compressed-tar",    ".tar.7z",   N_("Tar compressed with 7z (.tar.7z)"), FALSE, TRUE },
+	{ "application/x-tarz",                 ".tar.Z",    N_("Tar compressed with compress (.tar.Z)"), FALSE, TRUE },
+	{ "application/x-stuffit",              ".sit",      NULL, FALSE, TRUE },
+	{ "application/x-war",                  ".war",      N_("War (.war)"), TRUE, TRUE },
+	{ "application/zip",                    ".zip",      N_("Zip (.zip)"), TRUE, TRUE },
+	{ "application/x-zoo",                  ".zoo",      N_("Zoo (.zoo)"), FALSE, TRUE },
+	{ "application/x-7z-compressed",        ".7z",       N_("7-Zip (.7z)"), TRUE, TRUE },
 	{ NULL, NULL, NULL, FALSE, FALSE }
 };
 
@@ -138,8 +141,9 @@ FrExtensionType file_ext_type[] = {
 	{ ".tar.gz", "application/x-compressed-tar" },
 	{ ".tar.lzma", "application/x-lzma-compressed-tar" },
 	{ ".tar.lzo", "application/x-lzop-compressed-tar" },
-	{ ".tar.Z", "application/x-compressed-tar" },
-	{ ".taz", "application/x-compressed-tar" },
+	{ ".tar.7z", "application/x-7z-compressed-tar" },
+	{ ".tar.Z", "application/x-tarz" },
+	{ ".taz", "application/x-tarz" },
 	{ ".tbz", "application/x-bzip-compressed-tar" },
 	{ ".tbz2", "application/x-bzip-compressed-tar" },
 	{ ".tgz", "application/x-compressed-tar" },
@@ -147,7 +151,7 @@ FrExtensionType file_ext_type[] = {
 	{ ".tzo", "application/x-lzop-compressed-tar" },
 	{ ".war", "application/x-war" },
 	{ ".z", "application/x-gzip" },
-	{ ".Z", "application/x-gzip" },
+	{ ".Z", "application/x-compress" },
 	{ ".zip", "application/zip" },
 	{ ".zoo", "application/x-zoo" }
 };
@@ -155,6 +159,7 @@ FrExtensionType file_ext_type[] = {
 int single_file_save_type[32];
 int save_type[32];
 int open_type[32];
+int create_type[32];
 
 FrCommandDescription command_desc[] = {
 	{ "tar",        "application/x-tar", TRUE, TRUE },
@@ -169,7 +174,7 @@ FrCommandDescription command_desc[] = {
 	{ "ar",         "application/x-deb", TRUE, FALSE },
 	{ "arj",        "application/x-arj", TRUE, TRUE },
 	{ "bzip2",      "application/x-bzip1", TRUE, FALSE },
-	{ "compress",   "application/x-gzip", TRUE, TRUE },
+	{ "compress",   "application/x-compress", TRUE, TRUE },
 	{ "cpio",       "application/x-cpio", TRUE, FALSE },
 	{ "isoinfo",    "application/x-cd-image", TRUE, FALSE },
 	{ "zip",        "application/x-ear", TRUE, TRUE },
@@ -180,7 +185,7 @@ FrCommandDescription command_desc[] = {
 	{ "lzma",       "application/x-lzma", TRUE, TRUE },
 	{ "lzop",       "application/x-lzop", TRUE, TRUE },
 	{ "rpm2cpio",   "application/x-rpm", TRUE, FALSE },
-	{ "uncompress", "application/x-gzip", TRUE, FALSE },
+	{ "uncompress", "application/x-compress", TRUE, FALSE },
 	{ "unstuff",    "application/x-stuffit", TRUE, FALSE },
 	{ "zoo",        "application/x-zoo", TRUE, TRUE },
 	{ "7za",        "application/x-7z-compressed", TRUE, TRUE },
@@ -188,12 +193,15 @@ FrCommandDescription command_desc[] = {
 };
 
 FrCommandDescription tar_command_desc[] = {
-	{ "gzip",      "application/x-compressed-tar", TRUE, TRUE },
-	{ "bzip2",     "application/x-bzip-compressed-tar", TRUE, TRUE },
+	{ "gzip",       "application/x-compressed-tar", TRUE, TRUE },
+	{ "bzip2",      "application/x-bzip-compressed-tar", TRUE, TRUE },
 	/*{ "bzip",     "application/x-bzip1-compressed-tar", FALSE, TRUE },*/
-	{ "lzma",      "application/x-lzma-compressed-tar", TRUE, TRUE },
-	{ "lzop",      "application/x-lzop-compressed-tar", TRUE, TRUE },
-	{ "compress",  "application/x-compressed-tar", TRUE, TRUE }
+	{ "lzma",       "application/x-lzma-compressed-tar", TRUE, TRUE },
+	{ "lzop",       "application/x-lzop-compressed-tar", TRUE, TRUE },
+	{ "compress",   "application/x-tarz", FALSE, TRUE },
+	{ "uncompress", "application/x-tarz", TRUE, FALSE },
+	{ "7za",        "application/x-7z-compressed-tar", FALSE, TRUE },
+	{ "7zr",        "application/x-7z-compressed-tar", FALSE, TRUE }
 };
 
 
@@ -377,8 +385,8 @@ migrate_dir_from_to (const char *from_dir,
 		char *e1;
 		char *e2;
 
-		e1 = shell_escape (from_path);
-		e2 = shell_escape (to_path);
+		e1 = g_shell_quote (from_path);
+		e2 = g_shell_quote (to_path);
 		line = g_strdup_printf ("mv -f %s %s", e1, e2);
 		g_free (e1);
 		g_free (e2);
@@ -407,8 +415,8 @@ migrate_file_from_to (const char *from_file,
 		char *e1;
 		char *e2;
 
-		e1 = shell_escape (from_path);
-		e2 = shell_escape (to_path);
+		e1 = g_shell_quote (from_path);
+		e2 = g_shell_quote (to_path);
 		line = g_strdup_printf ("mv -f %s %s", e1, e2);
 		g_free (e1);
 		g_free (e2);
@@ -591,8 +599,10 @@ register_commands (void)
 			  "application/x-tar", FR_COMMAND_CAP_ALL,
 			  "application/x-compressed-tar", FR_COMMAND_CAP_ALL,
 			  "application/x-bzip-compressed-tar", FR_COMMAND_CAP_ALL,
+			  "application/x-tarz", FR_COMMAND_CAP_ALL,
 			  "application/x-lzma-compressed-tar", FR_COMMAND_CAP_ALL,
 			  "application/x-lzop-compressed-tar", FR_COMMAND_CAP_ALL,
+			  "application/x-7z-compressed-tar", FR_COMMAND_CAP_WRITE | FR_COMMAND_CAP_ARCHIVE_MANY_FILES,
 			  NULL);
 	register_command (FR_TYPE_COMMAND_UNSTUFF,
 			  "application/x-stuffit", FR_COMMAND_CAP_READ | FR_COMMAND_CAP_ARCHIVE_MANY_FILES,
@@ -682,7 +692,7 @@ static void
 compute_supported_archive_types (void)
 {
 	int i, j;
-	int sf_i = 0, s_i = 0, o_i = 0;
+	int sf_i = 0, s_i = 0, o_i = 0, c_i = 0;
 	int idx;
 	
 	for (i = 0; i < G_N_ELEMENTS (command_desc); i++) {
@@ -700,9 +710,16 @@ compute_supported_archive_types (void)
 					
 				idx = get_mime_type_index (comm_desc_2.mime_type);
 				if (idx >= 0) {
-					open_type[o_i++] = idx;
-					save_type[s_i++] = idx;
-					single_file_save_type[sf_i++] = idx;
+					if (comm_desc_2.can_open)
+						open_type[o_i++] = idx;
+					if (comm_desc_2.can_save) {
+						if (mime_type_desc[idx].supports_many_files) {
+							save_type[s_i++] = idx;
+							if (comm_desc_2.can_open)
+								create_type[c_i++] = idx;	
+						}
+						single_file_save_type[sf_i++] = idx;
+					}
 				}
 			}
 		}
@@ -711,16 +728,21 @@ compute_supported_archive_types (void)
 		if (idx >= 0) {
 			if (comm_desc.can_open)
 				open_type[o_i++] = idx;
-			if (comm_desc.can_save && mime_type_desc[idx].supports_many_files)
-				save_type[s_i++] = idx;
-			if (comm_desc.can_save)
+			if (comm_desc.can_save) {
+				if (mime_type_desc[idx].supports_many_files) {
+					save_type[s_i++] = idx;
+					if (comm_desc.can_open)
+						create_type[c_i++] = idx;
+				}
 				single_file_save_type[sf_i++] = idx;
+			}
 		}
 	}
 
 	open_type[o_i++] = -1;
 	save_type[s_i++] = -1;
 	single_file_save_type[sf_i++] = -1;
+	create_type[s_i++] = -1;
 }
 
 
@@ -849,11 +871,6 @@ prepare_app (void)
 					       FR_BATCH_ACTION_QUIT,
 					       NULL,
 					       NULL);
-
-		if (eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE))
-			/* open the destination folder only if we are 
-			 * extracting a single archive. */					       
-			fr_window_view_folder_after_extract (FR_WINDOW (window), (i - 1 == 1));
 					       
 		fr_window_start_batch (FR_WINDOW (window));
 	}
